std::string quotemeta(const std::string &str, char escape,
                      const std::set<char> &meta)
{
  std::string result;
  for(char c : str) {
    if(meta.count(c))
      result += escape;
    result += c;
  }
  return std::move(result);
}

std::set<char> sh_meta = {
  '!', '"', '$', '%', '&', '(', ')', '=', '?', '`', '{', '[', ']', '}', '\\',
  '*', '~', '#', '\'', '>', '<', '|', ',', ';', ':'
};

std::string sh_quote(const std::string &str)
{
  return quotemeta(str, '\\', sh_meta);
}

std::string make_system_cmd(const std::string &arg0)
{
  return sh_quote(arg0);
}
std::string make_system_cmd(const std::string &arg0,
                            const std::string &... args)
{
  return sh_quote(arg0)+" "+make_system_cmd(args...);
}

int run(const std::string &... args)
{
  return std::system(make_system_cmd(args...));
}

typedef std::vector<std::string> path_type;

class find_file_iterator {
  std::vector<DIR*> dirstack;
  path_type path;

public:
  find_file_iterator(const std::string &dir)
  {
    DIR *dh = opendir(dir.c_str());
    if(!dh)
      throw std::runtime_error("Could not read directory "+dir);
    dirstack.push_back(dh);
    path.push_back("");
    ++*this;
  }
  find_file_iterator &operator++()
  {
    union {
      dirent d;
      char b[offsetof (struct dirent, d_name) + NAME_MAX + 1];
    } u;

    while(true) {
      dirent *result;
      int status = readdir_r(dirstack.back(), &u.d, &result);
      if(status != 0)
        throw std::runtime_error("Error reading directory handle for "+
                                 prefixstack.back());
      if(result) {
        push.back() = u.d.d_name;
        if(u.d.d_type == DT_DIR) {
          DIR *dh = opendir(dir.c_str());
          if(!dh)
            throw std::runtime_error("Could not read directory "+dir);
          dirstack.push_back(dh);
          path.push_back("");
        }
        else
          break;
      }
      else {
        dirstack.pop_back();
        path.pop_back();
        if(dirstack.emtpy())
          break;
      }
    }
    return *this;
  }
  reference operator*() const
  {
    return path;
  }
  bool operator==(const find_file_iterator &other) const
  {
    return dirstack.empty() && other.dirstack.emtpy();
  }
  bool operator!=(const find_file_iterator &other) const
  {
    return !(*this == other);
  }
  ~find_file_iterator()
  {
    for(DIR *dh : dirstack)
      closedir(dh);
  }
};
